import os
import pydicom
import matplotlib.pyplot as plt
from PIL import Image
import numpy as np

plt.rcParams['font.family'] = 'Microsoft YaHei'

DICOM_PATH = os.path.join("D:\\", "Data", "Developing", "PanHuiQiong")


def draw_pixel_value_distribution(dcm, xmin=None, xmax=None, threshold=None, save_image=False, is_print=True,
                                  show=True):
    # dcm = pydicom.dcmread(dcm_file)
    pixel_array = dcm.pixel_array.astype(int)  # 获取CT数据，转换为浮点数类型

    pixels = pixel_array.reshape(-1)
    unique_elements, counts = np.unique(pixels, return_counts=True)
    for i in range(1, len(unique_elements)):
        if unique_elements[i] < unique_elements[i - 1]:
            raise Exception("np.unique得到无序列表")

    if xmin:
        xmin = int(xmin)
        less_than_xmin = np.where(unique_elements < xmin, True, False)
        _, c = np.unique(less_than_xmin, return_counts=True)
        imin = c[1]
    else:
        imin = 0

    if xmax:
        xmax = int(xmax)
        more_than_xmax = np.where(unique_elements > xmax, True, False)
        _, c = np.unique(more_than_xmax, return_counts=True)
        imax = -c[1] if len(c) == 2 else None
    else:
        imax = None

    print(f"最小值:{unique_elements[0]}, 最大值:{unique_elements[-1]}")
    print(f"imin:{imin}, imax:{imax}")
    unique_elements, counts = unique_elements[imin:imax], counts[imin:imax]
    # 绘制曲线
    plt.plot(unique_elements, counts)
    # 绘制高于threshold的异常点
    if threshold:
        threshold = int(threshold)
        stack_arr = np.stack((unique_elements, counts), axis=1)
        stack_arr = stack_arr[stack_arr[:, 1] >= threshold]
        threshold_elements, threshold_counts = stack_arr[:, 0], stack_arr[:, 1]
        plt.scatter(threshold_elements, threshold_counts, color='red', marker='o')
        for x, y in stack_arr:
            plt.text(x, y, f'({x}, {y})', ha='center', va='bottom', color='r')
    plt.title('统计不同Pixel value的像素个数的分布')
    plt.xlabel('Pixel value（Hu值）')
    plt.ylabel('像素个数')
    plt.xlim(unique_elements[0], unique_elements[-1] + 1)
    plt.ylim(0, np.max(counts) + 1)

    if save_image is True:
        (file, ext) = os.path.splitext(dcm_file)
        (path, filename) = os.path.split(file)

        scale = (256 * 256) / np.max(unique_elements)
        image = Image.fromarray((pixel_array * scale).astype(int))
        image.save(f"{filename}.png")

    # 打印出关键位置，查看具体异常像素值是哪几个
    if is_print is True:
        print("# 打印出关键位置，查看具体异常像素值是哪几个")
        for i, v in enumerate(unique_elements):
            c = counts[i]
            if threshold:
                if c >= int(threshold):
                    print(f"| \033[7;34;34m{v}\033[0m\t\033[31m{c}\033[0m\t", end=" ")
                else:
                    print(f"| {v}\t{c}\t", end=" ")
            else:
                c_color = 38 if c <= 10 else 32 if c < 1000 else 33 if c < 2000 else 31
                v_bg = "1;38;38m" if c <= 10 else "7;34;34m"
                print(f"| \033[{v_bg}{v}\033[0m\t\033[{c_color}m{c}\033[0m\t", end=" ")
            if (i + 1) % 10 == 0:
                print("|")

    if show is not True:
        # 获取FigureCanvas对象
        canvas = plt.gcf().canvas
        canvas.draw()
        # 获取绘图的渲染器
        renderer = canvas.get_renderer()
        # 渲染图形为RGB字节流
        image_rgb = renderer.tostring_rgb()
        # 将RGB字节流转换为NumPy数组
        width, height = canvas.get_width_height()
        image_array = np.frombuffer(image_rgb, dtype=np.uint8).reshape(height, width, 3)
        # 在此可以使用image_array，如保存到文件、进行进一步的处理等
        return image_array
    else:
        plt.show()


# ----------------------------------------------------------------------------------------------------------------------
if __name__ == "__main__":
    dcm_file = os.path.join(DICOM_PATH, "Pan Hui Qiong0280.dcm")
    # dicoms_to_nii(DICOM_PATH, "test.nii.gz")
    image_array = draw_pixel_value_distribution(dcm_file=dcm_file, show=False)
    print(type(image_array))
    image = Image.fromarray(image_array)
    image.save("testtest.png")



from PyQt5.QtGui import QImage


def pixel_array_2_qimage(pixel_array):
    if len(pixel_array.shape) == 3:
        height, width, channel = pixel_array.shape
        bytes_per_line = channel * width
        qimage = QImage(pixel_array.data, width, height, bytes_per_line, QImage.Format_RGB888)
    elif len(pixel_array.shape) == 2:
        height, width = pixel_array.shape
        bytes_per_line = width
        qimage = QImage(pixel_array.data, width, height, bytes_per_line, QImage.Format_Grayscale8)
    else:
        raise Exception(f"不支持的像素数组: pixel_array.shape={pixel_array.shape}")
    return qimage



import os
import pydicom
import numpy as np
import dicom2nifti
from PIL import Image

DICOM_PATH = os.path.join("D:\\", "Data", "Developing", "PanHuiQiong")


def get_dicoms(dicom_path):
    # Step1. 读取目录下的所有dicom文件
    dicoms = []
    for dcm in os.listdir(dicom_path):
        if dcm.endswith('.dcm'):
            dicoms.append(dcm)

    dicoms.sort()
    dcms = []
    for i in range(len(dicoms)):
        dcm = pydicom.dcmread(os.path.join(dicom_path, dicoms[i]))
        dcms.append(dcm)

    return dcms


def change_pixel_value2(dcm, value=None, when_positive=None, when_negative=None):
    """
    将CT数据中的某个像素值替换为另一个像素值
    如果像素值等于value，则替换为when_positive。如果像素值不等于value，则替换为when_negative。
    如果when_positive或when_negative为None，则不替换。
    dcm = pydicom.dcmread(dcm_file)
    """
    if value is not None:
        arr = dcm.pixel_array
        pixel_array = np.where(arr == value, when_positive if when_positive is not None else arr,
                               when_negative if when_negative is not None else arr)
        dcm.PixelData = pixel_array.astype(np.uint16).tobytes()

    return dcm


def change_pixel_value(dcm, conditions=None, default=0):
    """

    :param dcm:
    :param conditions: 键值对，键为像素值，值为替换后的像素值
    :param default: 默认值
    :return:
    """
    arr = dcm.pixel_array
    hu_map = [arr == int(hu) for hu in conditions]
    overwrites = [int(v) for v in conditions.values()]
    pixel_array = np.select(hu_map, overwrites, default=default)
    dcm.PixelData = pixel_array.astype(np.uint16).tobytes()
    return dcm


def change_pixel_value_from_dcmfile(dcm_file, conditions=None, default=0):
    dcm = pydicom.dcmread(dcm_file)
    return change_pixel_value(dcm, conditions, default=default)


def crop_pixel_value(dcm_file, value=None, output_dir='.'):
    pixel_array = change_pixel_value_from_dcmfile(dcm_file=dcm_file, conditions={value:3000}, default=0)
    scale = (256 * 256) / np.max(pixel_array)
    pixel_array = pixel_array * scale

    # 将CT数据转换为图像
    (file, ext) = os.path.splitext(dcm_file)
    (path, filename) = os.path.split(file)
    image = Image.fromarray(pixel_array.astype(int))
    image.save(os.path.join(output_dir, f"{filename}.cropped.{value}.png"))


def dicoms_to_nii_from_path(dicoms_path, nii_file):
    dicom2nifti.dicom_series_to_nifti(dicoms_path, nii_file)


def dicoms_to_nii(dcms, nii_file):
    dicom2nifti.convert_dicom.dicom_array_to_nifti(dcms, nii_file)


# ----------------------------------------------------------------------------------------------------------------------
if __name__ == "__main__":
    # dicoms_to_nii(DICOM_PATH, "test.nii.gz")
    dcms = get_dicoms(DICOM_PATH)

    value, when_positive, when_negative = 3252, 2000, 0
    for dcm in dcms:
        pixel_array = dcm.pixel_array
        pixel_array = np.where(pixel_array == value, 2500, 0)
        dcm.PixelData = pixel_array.astype(np.uint16).tobytes()

    dicoms_to_nii(dcms, "test4.nii.gz")



import os
import numpy as np
from PIL import Image
import nibabel as nib
import pandas as pd
import sys
from PyQt5.QtWidgets import QApplication, QMainWindow, QPushButton, QFileDialog
from skimage import color
from skimage.segmentation import find_boundaries
import SimpleITK as sitk
import cv2

np.set_printoptions(threshold=np.inf)


def get_totalsegmentation_labels():
    df = pd.read_csv('../data/TotalSegmentationLabels.txt', delimiter='\t')
    return {row[0]: row[1] for row in df.to_numpy()}


def save_image_from_2darray(images, folder, normalization=False, labeled_layer_only=False):
    os.makedirs(folder, exist_ok=True)
    for i, img in enumerate(images):
        min, max = np.min(img), np.max(img)
        if labeled_layer_only and min == max:
            continue

        if normalization:
            scaled_array = (img - min) / (max - min) * 255
            scaled_array = scaled_array.astype(np.uint8)
        else:
            scaled_array = img
        # 创建PIL图像对象
        image = Image.fromarray(scaled_array)
        # 保存图像为JPEG格式
        image.save(f'{folder}/{i + 1}.jpg')


def seperate_masks(nii_file):
    nii_data = nib.load(nii_file)
    nii_array = nii_data.get_fdata()
    images = []
    labels = get_totalsegmentation_labels()

    segmentations = set()
    for i in range(nii_array.shape[2]):
        image = nii_array[:, :, i]
        image_ravel = image.ravel()
        for pixel in image_ravel:
            if pixel not in segmentations:
                segmentations.add(pixel)

    segmentations.remove(0.0)

    for label_no in segmentations:
        images = []
        for i in range(nii_array.shape[2]):
            image = nii_array[:, :, i]
            mask = np.isclose(image, label_no, atol=1e-8)
            image[~mask] = 0
            images.append(image)

        # print(f'../outputs/{label_no}-{labels[round(label_no)]}')
        save_image_from_2darray(images, f'../outputs/{label_no}-{labels[round(label_no)]}')


def test_nii():
    # seperate_masks('../data/zqf_huge.nii.gz')

    # 读取NIfTI文件
    nii_file = '../data/hepaticvessel_022_label.nii.gz'

    nii_data = nib.load(nii_file)
    nii_array = nii_data.get_fdata()
    # print(nii_array[:, :, 237])
    print(nii_array.shape)
    print(nii_array.T.shape)
    images = []
    # for i in range(nii_array.shape[2]):
    #     image = nii_array[:, :, i]
    #     images.append(image)
    for img in nii_array.T:
        images.append(img)

    filename, extension = os.path.splitext(os.path.basename(nii_file))
    save_image_from_2darray(images, f'../outputs/{filename}', labeled_layer_only=True)


def normalize_nii(nii_file):
    # nii_file = '../data/hepaticvessel_022_label.nii.gz'
    nii_data = nib.load(nii_file)
    nii_array = nii_data.get_fdata().T


def boundary_mask_to_contour(mask):
    print(mask)


def test_boundary():
    layer = 26 - 1

    nii_file = '../data/hepaticvessel_050.nii.gz'
    # nii_array = np.fliplr(np.flipud(sitk.GetArrayFromImage(sitk.ReadImage(nii_file))[layer, :, :]))
    nii_array = np.rot90(sitk.GetArrayFromImage(sitk.ReadImage(nii_file))[layer, :, :], 2)

    pixel_min, pixel_max = np.min(nii_array), np.max(nii_array)
    print(pixel_min, pixel_max)
    scaled_array = (nii_array - pixel_min) / (pixel_max - pixel_min) * 255
    # gamma校正
    # gamma = 0.5
    # scaled_array = np.power(scaled_array, gamma)
    # pixel_min, pixel_max = np.min(scaled_array), np.max(scaled_array)
    # print(pixel_min, pixel_max)
    # scaled_array = (scaled_array - pixel_min) / (pixel_max - pixel_min) * 255

    scaled_array = scaled_array.astype(np.uint8)
    image = Image.fromarray(scaled_array)
    image.save('test.png')

    label_file = '../data/hepaticvessel_050_label.nii.gz'
    label_array = np.rot90(sitk.GetArrayFromImage(sitk.ReadImage(label_file))[layer, :, :], 2)

    label_1_mask = np.where(label_array == 1, 255 * 256, 0)
    label_2_mask = np.where(label_array == 2, 255 * 256, 0)
    label_1_mask_bound = find_boundaries(label_1_mask, connectivity=2)
    label_2_mask_bound = find_boundaries(label_2_mask, connectivity=2)

    label_rgb = color.gray2rgb(scaled_array)

    label_rgb[label_1_mask_bound] = [255, 0, 0]
    label_rgb[label_2_mask_bound] = [0, 255, 0]
    image = Image.fromarray(label_rgb)
    image.save('rgb.png')

    filename, extension = os.path.splitext(os.path.basename(nii_file))
    os.makedirs(f'../outputs/{filename}/patch', exist_ok=True)
    # 遍历边缘像素点
    for i, j in zip(*np.where(label_1_mask_bound)):
        # 以边缘像素点为中心选取 9x9 像素的方块
        x_size, y_size = nii_array.shape[0], nii_array.shape[1]

        margin = 32
        x_min = i - margin
        x_max = i + margin
        y_min = j - margin
        y_max = j + margin

        if x_min <= 0 or x_max >= x_size or y_min <= 0 or y_max >= y_size:
            continue

        # 提取方块区域
        patch = nii_array[x_min:x_max, y_min:y_max]
        patch_mask = np.where(label_1_mask[x_min:x_max, y_min:y_max] == 255 * 256, True, False)
        # patch_mask = label_1_mask_bound[x_min:x_max, y_min:y_max]

        scaled_array = (patch - pixel_min) / (pixel_max - pixel_min) * 255

        scaled_array = scaled_array.astype(np.uint8)

        # # 创建PIL图像对象

        scaled_array = color.gray2rgb(scaled_array)

        scaled_array_masked = scaled_array.copy()
        scaled_array_masked[patch_mask] = [255, 0, 0]

        # split = np.zeros((patch_mask.shape[1], 1, 3))
        # split.astype(np.uint8)
        merged = np.hstack((scaled_array, scaled_array_masked))

        image = Image.fromarray(scaled_array)
        scaled_array[patch_mask] = [255, 0, 0]
        image_masked = Image.fromarray(scaled_array)
        # # 保存图像为JPEG格式
        img = Image.fromarray(merged)

        img.save(f'../outputs/{filename}/patch/patch_{i}_{j}.png')
        # image.save(f'../outputs/{filename}/patch/patch_{i}_{j}.png')
        # image_masked.save(f'../outputs/{filename}/patch/patch_{i}_{j}_masked.png')


# //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
def nii2jpg_with_qt(labeled_layer_only=True):
    class MainWindow(QMainWindow):
        def __init__(self):
            super().__init__()
            self.initUI()

        def initUI(self):
            self.setWindowTitle("选择.nii文件")
            self.setGeometry(300, 300, 300, 200)

            button = QPushButton("打开", self)
            button.clicked.connect(self.openFileDialog)
            button.setGeometry(50, 50, 200, 50)

        def openFileDialog(self):
            options = QFileDialog.Options()
            options |= QFileDialog.DontUseNativeDialog
            fileName, _ = QFileDialog.getOpenFileName(self, "Select File", "", "All Files (*);;Text Files (*.txt)",
                                                      options=options)
            if fileName:
                print("Selected file:", fileName)

    app = QApplication(sys.argv)
    window = MainWindow()
    window.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    test_boundary()



import docker


client = docker.from_env()


class JLDocker():
    def __init__(self, env_file=None):
        if not env_file:
            self.client = docker.from_env()

    def _fit_in_line(self, line, length):
        diff = length - len(line)
        if diff < 0:
            return line[:length]
        else:
            return line + " "*diff

    def list(self, all=True):

        def _command(cmd, entrypoints):
            cmd = cmd if cmd else []
            entrypoints = entrypoints if entrypoints else []
            return " ".join(cmd + entrypoints)

        def _ports(ports):
            line = ""
            for k,v in ports.items():
                if not v:
                    line = line + ", " + k
                else:
                    line = line + ", " + v[0]['HostIp']+":" + v[0]['HostPort'] + "->" + k
            return line[2:]

        #for c in self.client.containers.list(all=all):
        #    print(c.attrs['NetworkSettings']['Networks'])

        NetworkName = 'bridge'
        return [[
                c.name,
                c.image.tags[0],
                c.status,
                _command(c.attrs['Config']['Cmd'], c.attrs['Config']['Entrypoint']),
                _ports(c.ports),
                c.attrs['NetworkSettings']['Networks'][NetworkName]['IPAddress']

            ]
            for c in self.client.containers.list(all=all)
        ]

    def ps(self, all=True):
        containers = self.list(all=all)
        space = lambda n : " "*n
        SPLIT = 7

        line = "-"*160
        header = "|  " + "name" + space(16-4-2) \
                +"|  " + "image" + space(16-5-2) \
                +"|  " + "ip address" + space(16-10-2) \
                +"|  " + "status" + space(10-6-2) \
                +"|  " + "command" + space(42-7-2) \
                +"|  " + "ports" + space(60-5-2-SPLIT) \
                +"|"

        print(line)
        print(header)
        print(line)
        for c in containers:
            name = "  " + self._fit_in_line(c[0], 12) + "  "
            image = "  " + self._fit_in_line(c[1], 12) + "  "
            ip = "  " + self._fit_in_line(c[5], 12) + "  "
            status = "  " + self._fit_in_line(c[2], 6) + "  "
            status = "\033[1;32m{}\033[0m".format(status) if "runnin" in status else status
            cmd = "  " + self._fit_in_line(c[3], 38) + "  "
            ports = "  " + self._fit_in_line(c[4], 50) + " "
            print(f"|{name}|{image}|{ip}|{status}|{cmd}|{ports}|")

        print(line)


if __name__ == "__main__":
    dock = JLDocker()
    dock.ps()



import os
import re
import sys
import json
import numpy as np
import pydicom
import matplotlib.pyplot as plt
from collections import OrderedDict

from PIL import Image
from PyQt5 import QtGui, QtCore
from PyQt5.QtWidgets import QApplication, QMainWindow, QWidget, QVBoxLayout, QHBoxLayout, QLabel, QLineEdit, \
    QPushButton, QFileDialog, QMessageBox
import dicom2nifti

from jilei.mimics import draw_pixel_value_distribution
from jilei.dicom import change_pixel_value, change_pixel_value_from_dcmfile, get_dicoms, dicoms_to_nii
from jilei.qt import pixel_array_2_qimage


def start():
    app = QApplication(sys.argv)
    window = MimicsToolWindow()
    window.show()
    sys.exit(app.exec_())


def message(title, text):
    msg = QMessageBox()
    msg.setWindowTitle(title)
    msg.setText(text)
    msg.setIcon(QMessageBox.Information)
    msg.addButton(QMessageBox.Ok)
    msg.exec()


def show(dcm, dcm_widget, plot_widget, xmin=None, xmax=None, threshold=None):
    if dcm is None:
        return
    image_array = draw_pixel_value_distribution(dcm=dcm, xmin=xmin, xmax=xmax, threshold=threshold,
                                                show=False)
    plot_widget.set_images(image_array=image_array)

    pixel_array = dcm.pixel_array
    scale = 255 / np.max(pixel_array)
    pixel_array = pixel_array * scale
    pixel_array = np.uint8(pixel_array)
    dcm_widget.set_images(image_array=pixel_array)


def show_label(labeled_widget, dcm_file, pixel_value):
    if dcm_file is None:
        return
    dcm = change_pixel_value_from_dcmfile(dcm_file=dcm_file, conditions={int(pixel_value): 255}, default=0)
    pixel_array = np.uint8(dcm.pixel_array)
    labeled_widget.set_images(image_array=pixel_array)


def convert_label(dcm_path=None, labels={}):
    """
    :param dcm_path:
    :param labels: 格式：[3252:airway;2491:vein;]
    :return:
    """
    if dcm_path is None or not os.path.exists(dcm_path):
        message("提示", "请指定正确的dicom文件夹路径")
        return

    if labels and labels[-1] != ';':
        labels = labels + ';'
    match = re.match(r"^(?:\d+,\d+,\w+;)+$", labels)
    if match is None:
        message("标签格式错误", "应符合 x,y,z; 的格式。例：3252,1,airway;2491,2,vein;")
        return

    # Key, Value, Class
    conditions = {kvc.split(",")[0]: kvc.split(",")[1] for kvc in labels[:-1].split(';')}
    labelnames = {kvc.split(",")[1]: kvc.split(",")[2] for kvc in labels[:-1].split(';')}

    arr = []
    dcms = get_dicoms(dicom_path=dcm_path)
    for dcm in dcms:
        dcm = change_pixel_value(dcm, conditions=conditions, default=0)
        arr.append(dcm)

    os.makedirs(os.path.join(dcm_path, 'dist'), exist_ok=True)
    dicoms_to_nii(arr, os.path.join(dcm_path, 'dist', "MSD.nii.gz"))

    # 保存标签名称
    with open(os.path.join(dcm_path, 'dist', "dataset.json"), 'w') as fp:
        labelnames["0"] = "background"
        labelnames = OrderedDict(sorted(labelnames.items()))
        datasetjson = {"labels": labelnames}
        modality = dcms[0].Modality
        if modality == "CT":
            datasetjson["modality"] = {"CT"}
        elif modality == "MR":
            datasetjson["modality"] = {"MRI"}
        else:
            datasetjson["modality"] = {modality}
        json_str = json.dumps(datasetjson, indent=4)
        fp.write(json_str)

    message("提示", "转换完成")


def convert_original(dcm_path):
    if dcm_path is None or not os.path.exists(dcm_path):
        message("提示", "请指定正确的dicom文件夹路径")
        return

    dcms = get_dicoms(dicom_path=dcm_path)
    os.makedirs(os.path.join(dcm_path, 'dist'), exist_ok=True)
    dicoms_to_nii(dcms, os.path.join(dcm_path, 'dist', "ORI.nii.gz"))


class ImageWidget(QWidget):
    def __init__(self):
        super().__init__()
        self.layout = QHBoxLayout()
        self.image_label = QLabel()
        self.image_label.setAlignment(QtCore.Qt.AlignCenter)  # 图片居中显示
        self.layout.addWidget(self.image_label)
        self.setLayout(self.layout)

    def set_images(self, image_path=None, image_array=None):
        if image_path is not None:
            pixmap = QtGui.QPixmap(image_path)
            self.image_label.setPixmap(pixmap)
            self.image_label.setScaledContents(True)  # 按比例缩放图片
        elif image_array is not None:
            qimage = pixel_array_2_qimage(image_array)
            pixmap = QtGui.QPixmap.fromImage(qimage)
            self.image_label.setPixmap(pixmap)
            self.image_label.setScaledContents(True)  # 按比例缩放图片


class MimicsToolWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self.dcm = None
        self.dcm_file = None
        self.dcm_path = None
        self.dcm_path_labeled = None
        self.labels = None

        # 创建主窗口
        self.setWindowTitle("积垒AI工具 - Mimics Mask转换为MSD(Medical Segmentation Decthlon) nii dataset")
        self.setGeometry(100, 100, 800, 600)

        """ 布局图
        |----------+----------|
        |          |          |
        |    LT    |    RT    |
        |          |          |
        |----------+----------|
        |          |          |
        |    LB    |    RB    |
        |          |          |
        |----------+----------|
        """

        """ ############################################################################################################
                                                        [M] 创建主布局
        ################################################################################################################
        """
        main_layout = QHBoxLayout()

        # --------------------------------------------------------------------------------------------------------------
        # - [L] 创建左侧布局] --------------------------------------------------------------------------------------------
        # --------------------------------------------------------------------------------------------------------------
        left_layout = QVBoxLayout()
        # -- [L] 创建左上角、左下角 ---------------------------------------------------------------------------------------
        lt_widget = ImageWidget()
        lb_widget = ImageWidget()
        # 设置大小
        lt_widget.setFixedSize(512, 512)
        lb_widget.setFixedSize(512, 512)
        # 将左上角、左下角和右下角区域添加到左侧布局中
        left_layout.addWidget(lt_widget)
        left_layout.addWidget(lb_widget)

        # --------------------------------------------------------------------------------------------------------------
        # - [R] 创建右侧布局 --------------------------------------------------------------------------------------------
        # --------------------------------------------------------------------------------------------------------------
        right_layout = QVBoxLayout()

        # -- [R] 创建右上角、右下角 ---------------------------------------------------------------------------------------
        rt_widget = QWidget()
        rb_widget = ImageWidget()
        rt_widget.setFixedSize(512, 512)
        rb_widget.setFixedSize(512, 512)

        # 测试
        test = True
        if test == True:
            lt_widget.setStyleSheet("background-color: #E12E4B;")
            lb_widget.setStyleSheet("background-color: #F9E54E;")
            # rt_widget.setStyleSheet("background-color: blue;")
            rb_widget.setStyleSheet("background-color: #5BBDC8;")

        """ ############################################################################################################
                                                     [RT] 右上角布局
        ################################################################################################################
        """
        rt_layout = QVBoxLayout(rt_widget)
        rt_layout.setSpacing(10)

        # --------------------------------------------------------------------------------------------------------------
        # [RT-1] DICOM测试
        # --------------------------------------------------------------------------------------------------------------
        rt_1_widget = QWidget()
        rt_1_layout = QVBoxLayout(rt_1_widget)
        # 第一部分：选择单张dicom进行测试，观察Hu值
        r1_title_label = QLabel("步骤一：先选取一张包含所有标签的DICOM, 观察CT异常值都有哪些")
        r1_title_label.setStyleSheet("color:rgb(5,58,58)")
        r1_title_label.setFont(QtGui.QFont("Roman times", 10, QtGui.QFont.Bold))
        self.rt_1_dicom_file_label = QLabel("选择Dicom文件: ")
        rt_1_button = QPushButton("打开DICOM文件")
        rt_1_button.clicked.connect(lambda: self.select_file(lb_widget))
        # 第二行：范围值输入框
        rt_1_range_layout = QHBoxLayout()
        rt_1_range_layout_label = QLabel("Hu范围值:")
        self.rt_1_min_input = QLineEdit()
        self.rt_1_max_input = QLineEdit()
        rt_1_range_layout_threshold_label1 = QLabel(" Count阈值:")
        self.rt_1_threshold_input = QLineEdit()
        rt_1_range_layout_threshold_label2 = QLabel("个")
        rt_1_range_layout.addWidget(rt_1_range_layout_label)
        rt_1_range_layout.addWidget(self.rt_1_min_input)
        rt_1_range_layout.addWidget(self.rt_1_max_input)
        rt_1_range_layout.addWidget(rt_1_range_layout_threshold_label1)
        rt_1_range_layout.addWidget(self.rt_1_threshold_input)
        rt_1_range_layout.addWidget(rt_1_range_layout_threshold_label2)
        show_figure_button = QPushButton("绘制")
        show_figure_button.clicked.connect(
            lambda: show(dcm=self.dcm, dcm_widget=lb_widget, plot_widget=lt_widget, xmin=self.rt_1_min_input.text(),
                         xmax=self.rt_1_max_input.text(), threshold=self.rt_1_threshold_input.text()))

        rt_1_layout.addWidget(r1_title_label)
        rt_1_layout.addWidget(self.rt_1_dicom_file_label)
        rt_1_layout.addWidget(rt_1_button)
        rt_1_layout.addLayout(rt_1_range_layout)
        rt_1_layout.addWidget(show_figure_button)

        # --------------------------------------------------------------------------------------------------------------
        # [RT-2] 标签高亮：单独显示标签
        # --------------------------------------------------------------------------------------------------------------
        rt_2_widget = QWidget()
        rt_2_layout = QVBoxLayout(rt_2_widget)
        rt_2_title_label = QLabel("步骤二：查看Hu异常值是哪个部位的标签")
        rt_2_title_label.setStyleSheet("color:rgb(5,58,58)")
        rt_2_title_label.setFont(QtGui.QFont("Roman times", 10, QtGui.QFont.Bold))

        rt_2_number_layout = QHBoxLayout()
        rt_2_number_input = QLineEdit()
        rt_2_number_button = QPushButton("单独显示标签")
        rt_2_number_button.clicked.connect(
            lambda: show_label(labeled_widget=rb_widget, dcm_file=self.dcm_file, pixel_value=rt_2_number_input.text()))
        rt_2_number_layout.addWidget(rt_2_number_input)
        rt_2_number_layout.addWidget(rt_2_number_button)

        rt_2_layout.addWidget(rt_2_title_label)
        rt_2_layout.addLayout(rt_2_number_layout)

        # --------------------------------------------------------------------------------------------------------------
        # [RT-3] 数据制作：将标签导出为MSD nii数据集
        # --------------------------------------------------------------------------------------------------------------
        rt_3_widget = QWidget()
        rt_3_layout = QVBoxLayout(rt_3_widget)

        rt_3_title_label1 = QLabel("步骤三：将Mimics导出标签(多个dicom)转换为nii")
        rt_3_title_label2 = QLabel("格式: 3252,1,airway;2491,2,vein; ")
        rt_3_title_label1.setStyleSheet("color:rgb(5,58,58)")
        rt_3_title_label2.setStyleSheet("color:rgb(5,58,58)")
        rt_3_title_label1.setFont(QtGui.QFont("Roman times", 10, QtGui.QFont.Bold))

        rt_3_path_layout = QHBoxLayout()
        self.rt_3_path_label = QLabel("选择Dicom文件目录: ")
        rt_3_path_button = QPushButton("打开目录")
        rt_3_path_button.clicked.connect(self.select_label_path)
        rt_3_path_layout.addWidget(self.rt_3_path_label)
        rt_3_path_layout.addWidget(rt_3_path_button)

        rt_3_convert_layout = QHBoxLayout()
        self.rt_3_text_input = QLineEdit()
        self.rt_3_convert_button = QPushButton("开始转换")
        self.rt_3_convert_button.clicked.connect(
            lambda: convert_label(self.dcm_path_labeled, self.rt_3_text_input.text()))
        rt_3_convert_layout.addWidget(self.rt_3_text_input)
        rt_3_convert_layout.addWidget(self.rt_3_convert_button)

        rt_3_layout.addWidget(rt_3_title_label1)
        rt_3_layout.addWidget(rt_3_title_label2)
        rt_3_layout.addLayout(rt_3_path_layout)
        rt_3_layout.addLayout(rt_3_convert_layout)

        # --------------------------------------------------------------------------------------------------------------
        # [RT-4] 将原始数据也制作为单个nii文件
        # --------------------------------------------------------------------------------------------------------------
        rt_4_widget = QWidget()
        rt_4_layout = QVBoxLayout(rt_4_widget)

        rt_4_title_label = QLabel("步骤四：选择目录，将多个原始DICOM文件转换为单个.nii")
        rt_4_title_label.setStyleSheet("color:rgb(5,58,58)")
        rt_4_title_label.setFont(QtGui.QFont("Roman times", 10, QtGui.QFont.Bold))

        rt_4_path_layout = QHBoxLayout()
        self.rt_4_path_label = QLabel("选择Dicom文件目录: ")
        rt_4_path_button = QPushButton("打开目录")
        rt_4_path_button.clicked.connect(self.select_path)
        rt_4_path_layout.addWidget(self.rt_4_path_label)
        rt_4_path_layout.addWidget(rt_4_path_button)

        rt_4_convert_layout = QHBoxLayout()
        self.rt_4_convert_button = QPushButton("开始转换")
        self.rt_4_convert_button.clicked.connect(lambda: convert_original(self.dcm_path))
        rt_4_convert_layout.addWidget(self.rt_4_convert_button)

        rt_4_layout.addWidget(rt_4_title_label)
        rt_4_layout.addLayout(rt_4_path_layout)
        rt_4_layout.addLayout(rt_4_convert_layout)

        # --------------------------------------------------------------------------------------------------------------
        # 将各个部件添加到右上角操作界面的布局中
        rt_layout.setAlignment(QtCore.Qt.AlignTop)
        rt_layout.addWidget(rt_1_widget)
        rt_layout.addWidget(rt_2_widget)
        rt_layout.addWidget(rt_3_widget)
        rt_layout.addWidget(rt_4_widget)

        # --------------------------------------------------------------------------------------------------------------
        # 组合各组件到父组件中
        # --------------------------------------------------------------------------------------------------------------
        # 将右上角操作界面添加到右侧布局中
        right_layout.addWidget(rt_widget)
        right_layout.addWidget(rb_widget)
        # 将左侧和右侧布局添加到主布局中
        main_layout.addLayout(left_layout)
        main_layout.addLayout(right_layout)
        # 创建主部件，并将主布局设置为主部件的布局
        main_widget = QWidget()
        # main_widget.setStyleSheet("background-color:rgb(5,58,58);")
        main_widget.setLayout(main_layout)
        # 设置主部件为主窗口的中心部件
        self.setCentralWidget(main_widget)

    def select_file(self, widget):
        file_dialog = QFileDialog()
        file_path, _ = file_dialog.getOpenFileName(self, "选择DICOM文件")
        if file_path:
            self.rt_1_dicom_file_label.setText(f"选择Dicom文件: {file_path}")
            self.dcm_file = file_path
            widget.set_images(file_path)
            self.dcm = pydicom.dcmread(self.dcm_file)

    def select_label_path(self):
        folder_path = QFileDialog.getExistingDirectory(self, "选择DICOM目录")
        self.rt_3_path_label.setText(folder_path)
        self.dcm_path_labeled = folder_path

    def select_path(self):
        folder_path = QFileDialog.getExistingDirectory(self, "选择DICOM目录")
        self.rt_4_path_label.setText(folder_path)
        self.dcm_path = folder_path


if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = MimicsToolWindow()
    window.show()
    sys.exit(app.exec_())



import pygltflib

# 读取GLB文件
glb_file_path = "../data/zqf.glb"
glb = pygltflib.GLTF2().load(glb_file_path)

# 获取场景
scene = glb.scenes[0]

# 获取场景中所有节点的位置信息
positions = []
for node_idx in scene.nodes:
    node = glb.nodes[node_idx]
    if node.mesh is not None:
        mesh = glb.meshes[node.mesh]
        for primitive in mesh.primitives:
            accessor_idx = primitive.attributes.POSITION
            accessor = glb.accessors[accessor_idx]
            buffer_view = glb.buffer_views[accessor.buffer_view]
            buffer = glb.buffer_views[buffer_view.buffer]
            position_data = buffer.data[buffer_view.byte_offset + accessor.byte_offset: buffer_view.byte_offset + accessor.byte_offset + accessor.count * accessor.data_type.itemsize]
            positions.extend(position_data)

# 计算几何中心
x_values = positions[::3]
y_values = positions[1::3]
z_values = positions[2::3]
center_x = sum(x_values) / len(x_values)
center_y = sum(y_values) / len(y_values)
center_z = sum(z_values) / len(z_values)

print("物体几何中心坐标：")
print("X坐标：", center_x)
print("Y坐标：", center_y)
print("Z坐标：", center_z)


import nibabel as nib

# 读取NIfTI文件
nifti_file_path = "../data/zqf.nii.gz"
nifti_image = nib.load(nifti_file_path)

origin = nifti_image.affine[:3, 3]

print("CT原点位置：")
print("X坐标：", origin[0])
print("Y坐标：", origin[1])
print("Z坐标：", origin[2])